@@ -105,7 +105,7 @@
  * with different configurations, by simply doing:
  *
  * |[<!-- language="C" -->
- *   GstHarness * h = gst_harness_new (h, "vp8dec");
+ *   GstHarness * h = gst_harness_new ("vp8dec");
  *   gst_harness_add_src_parse (h, "videotestsrc is-live=1 ! vp8enc", TRUE);
  * ]|
  *
@@ -126,6 +126,7 @@
 #endif
 
 #include "gstharness.h"
+#include "gstharnesslink.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -133,8 +134,12 @@
 
 static void gst_harness_stress_free (GstHarnessThread * t);
 
-#define HARNESS_KEY "harness"
+/* Keys used for storing and retrieving associations to pads and elements with
+ * g_object_set_data (): */
+/* The HARNESS_REF association inside a GstElement will keep track of how many
+ * GstHarness'es are attached to that element. */
 #define HARNESS_REF "harness-ref"
+
 #define HARNESS_LOCK(h) g_mutex_lock (&(h)->priv->priv_mutex)
 #define HARNESS_UNLOCK(h) g_mutex_unlock (&(h)->priv->priv_mutex)
 
@@ -172,9 +177,9 @@ struct _GstHarnessPrivate
   GstPad *sink_forward_pad;
   GstTestClock *testclock;
 
-  volatile gint recv_buffers;
-  volatile gint recv_events;
-  volatile gint recv_upstream_events;
+  gint recv_buffers;
+  gint recv_events;
+  gint recv_upstream_events;
 
   GAsyncQueue *buffer_queue;
   GAsyncQueue *src_event_queue;
@@ -182,6 +187,8 @@ struct _GstHarnessPrivate
 
   GstClockTime latency_min;
   GstClockTime latency_max;
+  gboolean is_live;
+
   gboolean has_clock_wait;
   gboolean drop_buffers;
   GstClockTime last_push_ts;
@@ -209,10 +216,16 @@ struct _GstHarnessPrivate
 static GstFlowReturn
 gst_harness_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
 {
-  GstHarness *h = g_object_get_data (G_OBJECT (pad), HARNESS_KEY);
-  GstHarnessPrivate *priv = h->priv;
+  GstHarness *h;
+  GstHarnessLink *link;
+  GstHarnessPrivate *priv;
   (void) parent;
+
+  if (!(link = gst_harness_pad_link_lock (pad, &h)))
+    return GST_FLOW_FLUSHING;
   g_assert (h != NULL);
+  priv = h->priv;
+
   g_mutex_lock (&priv->blocking_push_mutex);
   g_atomic_int_inc (&priv->recv_buffers);
 
@@ -231,31 +244,45 @@ gst_harness_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
   }
   g_mutex_unlock (&priv->blocking_push_mutex);
 
+  gst_harness_link_unlock (link);
   return GST_FLOW_OK;
 }
 
 static gboolean
 gst_harness_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
 {
-  GstHarness *h = g_object_get_data (G_OBJECT (pad), HARNESS_KEY);
-  GstHarnessPrivate *priv = h->priv;
+  GstHarness *h;
+  GstHarnessLink *link;
+  GstHarnessPrivate *priv;
   (void) parent;
+
+  if (!(link = gst_harness_pad_link_lock (pad, &h)))
+    return FALSE;
   g_assert (h != NULL);
+  priv = h->priv;
+
   g_atomic_int_inc (&priv->recv_upstream_events);
   g_async_queue_push (priv->src_event_queue, event);
+
+  gst_harness_link_unlock (link);
   return TRUE;
 }
 
 static gboolean
 gst_harness_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
 {
-  GstHarness *h = g_object_get_data (G_OBJECT (pad), HARNESS_KEY);
-  GstHarnessPrivate *priv = h->priv;
+  GstHarness *h;
+  GstHarnessLink *link;
+  GstHarnessPrivate *priv;
   gboolean ret = TRUE;
   gboolean forward;
-
-  g_assert (h != NULL);
   (void) parent;
+
+  if (!(link = gst_harness_pad_link_lock (pad, &h)))
+    return FALSE;
+  g_assert (h != NULL);
+  priv = h->priv;
+
   g_atomic_int_inc (&priv->recv_events);
 
   switch (GST_EVENT_TYPE (event)) {
@@ -287,6 +314,7 @@ gst_harness_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
   }
   HARNESS_UNLOCK (h);
 
+  gst_harness_link_unlock (link);
   return ret;
 }
 
@@ -359,23 +387,31 @@ gst_harness_negotiate (GstHarness * h)
     gst_harness_decide_allocation (h, caps);
     gst_caps_unref (caps);
   } else {
-    GST_FIXME_OBJECT (h, "Cannot negotiate allocation because caps is not set");
+    GST_FIXME_OBJECT (h->srcpad,
+        "Cannot negotiate allocation because caps is not set");
   }
 }
 
 static gboolean
 gst_harness_sink_query (GstPad * pad, GstObject * parent, GstQuery * query)
 {
-  GstHarness *h = g_object_get_data (G_OBJECT (pad), HARNESS_KEY);
-  GstHarnessPrivate *priv = h->priv;
+  GstHarness *h;
+  GstHarnessLink *link;
+  GstHarnessPrivate *priv;
   gboolean res = TRUE;
+  (void) parent;
+
+  if (!(link = gst_harness_pad_link_lock (pad, &h)))
+    return FALSE;
   g_assert (h != NULL);
+  priv = h->priv;
 
   // FIXME: forward all queries?
 
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_LATENCY:
-      gst_query_set_latency (query, TRUE, priv->latency_min, priv->latency_max);
+      gst_query_set_latency (query, priv->is_live, priv->latency_min,
+          priv->latency_max);
       break;
     case GST_QUERY_CAPS:
     {
@@ -452,20 +488,28 @@ gst_harness_sink_query (GstPad * pad, GstObject * parent, GstQuery * query)
       res = gst_pad_query_default (pad, parent, query);
   }
 
+  gst_harness_link_unlock (link);
   return res;
 }
 
 static gboolean
 gst_harness_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
 {
-  GstHarness *h = g_object_get_data (G_OBJECT (pad), HARNESS_KEY);
-  GstHarnessPrivate *priv = h->priv;
+  GstHarness *h;
+  GstHarnessLink *link;
+  GstHarnessPrivate *priv;
   gboolean res = TRUE;
+  (void) parent;
+
+  if (!(link = gst_harness_pad_link_lock (pad, &h)))
+    return FALSE;
   g_assert (h != NULL);
+  priv = h->priv;
 
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_LATENCY:
-      gst_query_set_latency (query, TRUE, priv->latency_min, priv->latency_max);
+      gst_query_set_latency (query, priv->is_live, priv->latency_min,
+          priv->latency_max);
       break;
     case GST_QUERY_CAPS:
     {
@@ -490,6 +534,7 @@ gst_harness_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
     default:
       res = gst_pad_query_default (pad, parent, query);
   }
+  gst_harness_link_unlock (link);
   return res;
 }
 
@@ -535,7 +580,7 @@ gst_harness_link_element_srcpad (GstHarness * h,
       element_srcpad_name);
   GstPadLinkReturn link;
   if (srcpad == NULL)
-    srcpad = gst_element_get_request_pad (h->element, element_srcpad_name);
+    srcpad = gst_element_request_pad_simple (h->element, element_srcpad_name);
   g_assert (srcpad);
   link = gst_pad_link (srcpad, h->sinkpad);
   g_assert_cmpint (link, ==, GST_PAD_LINK_OK);
@@ -554,7 +599,7 @@ gst_harness_link_element_sinkpad (GstHarness * h,
       element_sinkpad_name);
   GstPadLinkReturn link;
   if (sinkpad == NULL)
-    sinkpad = gst_element_get_request_pad (h->element, element_sinkpad_name);
+    sinkpad = gst_element_request_pad_simple (h->element, element_sinkpad_name);
   g_assert (sinkpad);
   link = gst_pad_link (h->srcpad, sinkpad);
   g_assert_cmpint (link, ==, GST_PAD_LINK_OK);
@@ -574,8 +619,8 @@ gst_harness_setup_src_pad (GstHarness * h,
   /* sending pad */
   h->srcpad = gst_pad_new_from_static_template (src_tmpl, "src");
   g_assert (h->srcpad);
-  g_object_set_data (G_OBJECT (h->srcpad), HARNESS_KEY, h);
 
+  gst_harness_pad_link_set (h->srcpad, h);
   gst_pad_set_query_function (h->srcpad, gst_harness_src_query);
   gst_pad_set_event_function (h->srcpad, gst_harness_src_event);
 
@@ -595,8 +640,8 @@ gst_harness_setup_sink_pad (GstHarness * h,
   /* receiving pad */
   h->sinkpad = gst_pad_new_from_static_template (sink_tmpl, "sink");
   g_assert (h->sinkpad);
-  g_object_set_data (G_OBJECT (h->sinkpad), HARNESS_KEY, h);
 
+  gst_harness_pad_link_set (h->sinkpad, h);
   gst_pad_set_chain_function (h->sinkpad, gst_harness_chain);
   gst_pad_set_query_function (h->sinkpad, gst_harness_sink_query);
   gst_pad_set_event_function (h->sinkpad, gst_harness_sink_event);
@@ -663,7 +708,7 @@ gst_pad_is_request_pad (GstPad * pad)
  *
  * MT safe.
  *
- * Returns: (transfer full): a #GstHarness, or %NULL if the harness could
+ * Returns: (transfer full) (nullable): a #GstHarness, or %NULL if the harness could
  * not be created
  *
  * Since: 1.8
@@ -683,6 +728,7 @@ gst_harness_new_empty (void)
   priv->last_push_ts = GST_CLOCK_TIME_NONE;
   priv->latency_min = 0;
   priv->latency_max = GST_CLOCK_TIME_NONE;
+  priv->is_live = TRUE;
   priv->drop_buffers = FALSE;
   priv->testclock = GST_TEST_CLOCK_CAST (gst_test_clock_new ());
 
@@ -717,15 +763,15 @@ gst_harness_new_empty (void)
  * gst_harness_add_element_full: (skip)
  * @h: a #GstHarness
  * @element: a #GstElement to add to the harness (transfer none)
- * @hsrc: (allow-none): a #GstStaticPadTemplate describing the harness srcpad.
+ * @hsrc: (nullable): a #GstStaticPadTemplate describing the harness srcpad.
  * %NULL will not create a harness srcpad.
- * @element_sinkpad_name: (allow-none): a #gchar with the name of the element
+ * @element_sinkpad_name: (nullable): a #gchar with the name of the element
  * sinkpad that is then linked to the harness srcpad. Can be a static or request
  * or a sometimes pad that has been added. %NULL will not get/request a sinkpad
  * from the element. (Like if the element is a src.)
- * @hsink: (allow-none): a #GstStaticPadTemplate describing the harness sinkpad.
+ * @hsink: (nullable): a #GstStaticPadTemplate describing the harness sinkpad.
  * %NULL will not create a harness sinkpad.
- * @element_srcpad_name: (allow-none): a #gchar with the name of the element
+ * @element_srcpad_name: (nullable): a #gchar with the name of the element
  * srcpad that is then linked to the harness sinkpad, similar to the
  * @element_sinkpad_name.
  *
@@ -798,15 +844,15 @@ gst_harness_add_element_full (GstHarness * h, GstElement * element,
 /**
  * gst_harness_new_full: (skip)
  * @element: a #GstElement to attach the harness to (transfer none)
- * @hsrc: (allow-none): a #GstStaticPadTemplate describing the harness srcpad.
+ * @hsrc: (nullable): a #GstStaticPadTemplate describing the harness srcpad.
  * %NULL will not create a harness srcpad.
- * @element_sinkpad_name: (allow-none): a #gchar with the name of the element
+ * @element_sinkpad_name: (nullable): a #gchar with the name of the element
  * sinkpad that is then linked to the harness srcpad. Can be a static or request
  * or a sometimes pad that has been added. %NULL will not get/request a sinkpad
  * from the element. (Like if the element is a src.)
- * @hsink: (allow-none): a #GstStaticPadTemplate describing the harness sinkpad.
+ * @hsink: (nullable): a #GstStaticPadTemplate describing the harness sinkpad.
  * %NULL will not create a harness sinkpad.
- * @element_srcpad_name: (allow-none): a #gchar with the name of the element
+ * @element_srcpad_name: (nullable): a #gchar with the name of the element
  * srcpad that is then linked to the harness sinkpad, similar to the
  * @element_sinkpad_name.
  *
@@ -814,7 +860,7 @@ gst_harness_add_element_full (GstHarness * h, GstElement * element,
  *
  * MT safe.
  *
- * Returns: (transfer full): a #GstHarness, or %NULL if the harness could
+ * Returns: (transfer full) (nullable): a #GstHarness, or %NULL if the harness could
  * not be created
  *
  * Since: 1.6
@@ -834,10 +880,10 @@ gst_harness_new_full (GstElement * element,
 /**
  * gst_harness_new_with_element: (skip)
  * @element: a #GstElement to attach the harness to (transfer none)
- * @element_sinkpad_name: (allow-none): a #gchar with the name of the element
+ * @element_sinkpad_name: (nullable): a #gchar with the name of the element
  * sinkpad that is then linked to the harness srcpad. %NULL does not attach a
  * sinkpad
- * @element_srcpad_name: (allow-none): a #gchar with the name of the element
+ * @element_srcpad_name: (nullable): a #gchar with the name of the element
  * srcpad that is then linked to the harness sinkpad. %NULL does not attach a
  * srcpad
  *
@@ -847,7 +893,7 @@ gst_harness_new_full (GstElement * element,
  *
  * MT safe.
  *
- * Returns: (transfer full): a #GstHarness, or %NULL if the harness could
+ * Returns: (transfer full) (nullable): a #GstHarness, or %NULL if the harness could
  * not be created
  *
  * Since: 1.6
@@ -863,10 +909,10 @@ gst_harness_new_with_element (GstElement * element,
 /**
  * gst_harness_new_with_padnames: (skip)
  * @element_name: a #gchar describing the #GstElement name
- * @element_sinkpad_name: (allow-none): a #gchar with the name of the element
+ * @element_sinkpad_name: (nullable): a #gchar with the name of the element
  * sinkpad that is then linked to the harness srcpad. %NULL does not attach a
  * sinkpad
- * @element_srcpad_name: (allow-none): a #gchar with the name of the element
+ * @element_srcpad_name: (nullable): a #gchar with the name of the element
  * srcpad that is then linked to the harness sinkpad. %NULL does not attach a
  * srcpad
  *
@@ -875,7 +921,7 @@ gst_harness_new_with_element (GstElement * element,
  *
  * MT safe.
  *
- * Returns: (transfer full): a #GstHarness, or %NULL if the harness could
+ * Returns: (transfer full) (nullable): a #GstHarness, or %NULL if the harness could
  * not be created
  *
  * Since: 1.6
@@ -897,9 +943,9 @@ gst_harness_new_with_padnames (const gchar * element_name,
 /**
  * gst_harness_new_with_templates: (skip)
  * @element_name: a #gchar describing the #GstElement name
- * @hsrc: (allow-none): a #GstStaticPadTemplate describing the harness srcpad.
+ * @hsrc: (nullable): a #GstStaticPadTemplate describing the harness srcpad.
  * %NULL will not create a harness srcpad.
- * @hsink: (allow-none): a #GstStaticPadTemplate describing the harness sinkpad.
+ * @hsink: (nullable): a #GstStaticPadTemplate describing the harness sinkpad.
  * %NULL will not create a harness sinkpad.
  *
  * Creates a new harness, like gst_harness_new_full(), except it
@@ -907,7 +953,7 @@ gst_harness_new_with_padnames (const gchar * element_name,
  *
  * MT safe.
  *
- * Returns: (transfer full): a #GstHarness, or %NULL if the harness could
+ * Returns: (transfer full) (nullable): a #GstHarness, or %NULL if the harness could
  * not be created
  *
  * Since: 1.6
@@ -934,7 +980,7 @@ gst_harness_new_with_templates (const gchar * element_name,
  *
  * MT safe.
  *
- * Returns: (transfer full): a #GstHarness, or %NULL if the harness could
+ * Returns: (transfer full) (nullable): a #GstHarness, or %NULL if the harness could
  * not be created
  *
  * Since: 1.6
@@ -1027,7 +1073,7 @@ gst_harness_add_parse (GstHarness * h, const gchar * launchline)
  *
  * MT safe.
  *
- * Returns: (transfer full): a #GstHarness, or %NULL if the harness could
+ * Returns: (transfer full) (nullable): a #GstHarness, or %NULL if the harness could
  * not be created
  *
  * Since: 1.6
@@ -1084,10 +1130,9 @@ gst_harness_teardown (GstHarness * h)
 
     /* Make sure our funcs are not called after harness is teared down since
      * they try to access this harness through pad data */
-    GST_PAD_STREAM_LOCK (h->srcpad);
+    gst_harness_pad_link_tear_down (h->srcpad);
     gst_pad_set_event_function (h->srcpad, NULL);
     gst_pad_set_query_function (h->srcpad, NULL);
-    GST_PAD_STREAM_UNLOCK (h->srcpad);
 
     gst_object_unref (h->srcpad);
   }
@@ -1102,11 +1147,10 @@ gst_harness_teardown (GstHarness * h)
 
     /* Make sure our funcs are not called after harness is teared down since
      * they try to access this harness through pad data */
-    GST_PAD_STREAM_LOCK (h->sinkpad);
+    gst_harness_pad_link_tear_down (h->sinkpad);
     gst_pad_set_chain_function (h->sinkpad, NULL);
     gst_pad_set_event_function (h->sinkpad, NULL);
     gst_pad_set_query_function (h->sinkpad, NULL);
-    GST_PAD_STREAM_UNLOCK (h->sinkpad);
 
     gst_object_unref (h->sinkpad);
   }
@@ -1384,7 +1428,7 @@ gst_harness_use_testclock (GstHarness * h)
  *
  * MT safe.
  *
- * Returns: (transfer full): a #GstTestClock, or %NULL if the testclock is not
+ * Returns: (transfer full) (nullable): a #GstTestClock, or %NULL if the testclock is not
  * present.
  *
  * Since: 1.6
@@ -1670,7 +1714,7 @@ gst_harness_push (GstHarness * h, GstBuffer * buffer)
  *
  * MT safe.
  *
- * Returns: (transfer full): a #GstBuffer or %NULL if timed out.
+ * Returns: (transfer full) (nullable): a #GstBuffer or %NULL if timed out.
  *
  * Since: 1.6
  */
@@ -1693,7 +1737,7 @@ gst_harness_pull (GstHarness * h)
 /**
  * gst_harness_pull_until_eos:
  * @h: a #GstHarness
- * @buf: (out) (transfer full): A #GstBuffer, or %NULL if EOS or timeout occures
+ * @buf: (out) (nullable) (transfer full): A #GstBuffer, or %NULL if EOS or timeout occures
  *   first.
  *
  * Pulls a #GstBuffer from the #GAsyncQueue on the #GstHarness sinkpad. The pull
@@ -1734,7 +1778,7 @@ gst_harness_pull_until_eos (GstHarness * h, GstBuffer ** buf)
  *
  * MT safe.
  *
- * Returns: (transfer full): a #GstBuffer or %NULL if no buffers are present in the #GAsyncQueue
+ * Returns: (transfer full) (nullable): a #GstBuffer or %NULL if no buffers are present in the #GAsyncQueue
  *
  * Since: 1.6
  */
@@ -1764,7 +1808,7 @@ gst_harness_try_pull (GstHarness * h)
  *
  * MT safe.
  *
- * Returns: (transfer full): a #GstBuffer or %NULL if timed out.
+ * Returns: (transfer full) (nullable): a #GstBuffer or %NULL if timed out.
  *
  * Since: 1.6
  */
@@ -1978,7 +2022,7 @@ gst_harness_get_last_pushed_timestamp (GstHarness * h)
 /**
  * gst_harness_push_event:
  * @h: a #GstHarness
- * @event: a #GstEvent to push
+ * @event: (transfer full): a #GstEvent to push
  *
  * Pushes an #GstEvent on the #GstHarness srcpad.
  *
@@ -2003,7 +2047,7 @@ gst_harness_push_event (GstHarness * h, GstEvent * event)
  *
  * MT safe.
  *
- * Returns: a #GstEvent or %NULL if timed out.
+ * Returns: (transfer full) (nullable): a #GstEvent or %NULL if timed out.
  *
  * Since: 1.6
  */
@@ -2024,7 +2068,7 @@ gst_harness_pull_event (GstHarness * h)
  *
  * MT safe.
  *
- * Returns: a #GstEvent or %NULL if no buffers are present in the #GAsyncQueue
+ * Returns: (transfer full) (nullable): a #GstEvent or %NULL if no buffers are present in the #GAsyncQueue
  *
  * Since: 1.6
  */
@@ -2078,7 +2122,7 @@ gst_harness_events_in_queue (GstHarness * h)
 /**
  * gst_harness_push_upstream_event:
  * @h: a #GstHarness
- * @event: a #GstEvent to push
+ * @event: (transfer full): a #GstEvent to push
  *
  * Pushes an #GstEvent on the #GstHarness sinkpad.
  *
@@ -2106,7 +2150,7 @@ gst_harness_push_upstream_event (GstHarness * h, GstEvent * event)
  *
  * MT safe.
  *
- * Returns: a #GstEvent or %NULL if timed out.
+ * Returns: (transfer full) (nullable): a #GstEvent or %NULL if timed out.
  *
  * Since: 1.6
  */
@@ -2127,7 +2171,7 @@ gst_harness_pull_upstream_event (GstHarness * h)
  *
  * MT safe.
  *
- * Returns: a #GstEvent or %NULL if no buffers are present in the #GAsyncQueue
+ * Returns: (transfer full) (nullable): a #GstEvent or %NULL if no buffers are present in the #GAsyncQueue
  *
  * Since: 1.6
  */
@@ -2219,16 +2263,34 @@ gst_harness_query_latency (GstHarness * h)
  */
 void
 gst_harness_set_upstream_latency (GstHarness * h, GstClockTime latency)
+{
+  g_return_if_fail (GST_CLOCK_TIME_IS_VALID (latency));
+
+  h->priv->latency_min = latency;
+}
+
+/**
+ * gst_harness_set_live:
+ * @h: a #GstHarness
+ * @is_live: %TRUE for live, %FALSE for non-live
+ *
+ * Sets the liveness reported by #GstHarness when receiving a latency-query.
+ * The default is %TRUE.
+ *
+ * Since: 1.20
+ */
+void
+gst_harness_set_live (GstHarness * h, gboolean is_live)
 {
   GstHarnessPrivate *priv = h->priv;
-  priv->latency_min = latency;
+  priv->is_live = is_live;
 }
 
 /**
  * gst_harness_get_allocator:
  * @h: a #GstHarness
- * @allocator: (out) (allow-none) (transfer none): the #GstAllocator used
- * @params: (out) (allow-none) (transfer full): the #GstAllocationParams of
+ * @allocator: (out) (optional) (nullable) (transfer none): the #GstAllocator used
+ * @params: (out) (optional): the #GstAllocationParams of
  *   @allocator
  *
  * Gets the @allocator and its @params that has been decided to use after an
@@ -2253,8 +2315,8 @@ gst_harness_get_allocator (GstHarness * h, GstAllocator ** allocator,
 /**
  * gst_harness_set_propose_allocator:
  * @h: a #GstHarness
- * @allocator: (allow-none) (transfer full): a #GstAllocator
- * @params: (allow-none) (transfer none): a #GstAllocationParams
+ * @allocator: (nullable) (transfer full): a #GstAllocator
+ * @params: (nullable) (transfer none): a #GstAllocationParams
  *
  * Sets the @allocator and @params to propose when receiving an allocation
  * query.
@@ -2278,7 +2340,7 @@ gst_harness_set_propose_allocator (GstHarness * h, GstAllocator * allocator,
  * gst_harness_add_propose_allocation_meta:
  * @h: a #GstHarness
  * @api: a metadata API
- * @params: (allow-none) (transfer none): API specific parameters
+ * @params: (nullable) (transfer none): API specific parameters
  *
  * Add api with params as one of the supported metadata API to propose when
  * receiving an allocation query.
@@ -2656,7 +2718,7 @@ gst_harness_sink_push_many (GstHarness * h, gint pushes)
  *
  * MT safe.
  *
- * Returns: (transfer full) (allow-none): a #GstElement or %NULL if not found
+ * Returns: (transfer full) (nullable): a #GstElement or %NULL if not found
  *
  * Since: 1.6
  */
@@ -2874,13 +2936,13 @@ gst_harness_thread_init (GstHarnessThread * t, GDestroyNotify freefunc,
 static void
 gst_harness_thread_free (GstHarnessThread * t)
 {
-  g_slice_free (GstHarnessThread, t);
+  g_free (t);
 }
 
 static void
 gst_harness_custom_thread_free (GstHarnessCustomThread * t)
 {
-  g_slice_free (GstHarnessCustomThread, t);
+  g_free (t);
 }
 
 static void
@@ -2890,7 +2952,7 @@ gst_harness_push_buffer_thread_free (GstHarnessPushBufferThread * t)
     gst_caps_replace (&t->caps, NULL);
     if (t->notify != NULL)
       t->notify (t->data);
-    g_slice_free (GstHarnessPushBufferThread, t);
+    g_free (t);
   }
 }
 
@@ -2900,7 +2962,7 @@ gst_harness_push_event_thread_free (GstHarnessPushEventThread * t)
   if (t != NULL) {
     if (t->notify != NULL)
       t->notify (t->data);
-    g_slice_free (GstHarnessPushEventThread, t);
+    g_free (t);
   }
 }
 
@@ -2910,7 +2972,7 @@ gst_harness_property_thread_free (GstHarnessPropThread * t)
   if (t != NULL) {
     g_free (t->name);
     g_value_unset (&t->value);
-    g_slice_free (GstHarnessPropThread, t);
+    g_free (t);
   }
 }
 
@@ -2939,7 +3001,7 @@ gst_harness_requestpad_thread_free (GstHarnessReqPadThread * t)
     gst_caps_replace (&t->caps, NULL);
 
     gst_harness_requestpad_release_pads (t);
-    g_slice_free (GstHarnessReqPadThread, t);
+    g_free (t);
   }
 }
 
@@ -3166,7 +3228,7 @@ gst_harness_stress_thread_stop (GstHarnessThread * t)
 /**
  * gst_harness_stress_custom_start: (skip)
  * @h: a #GstHarness
- * @init: (allow-none): a #GFunc that is called initially and only once
+ * @init: (nullable): a #GFunc that is called initially and only once
  * @callback: a #GFunc that is called as often as possible
  * @data: a #gpointer with custom data to pass to the @callback function
  * @sleep: a #gulong specifying how long to sleep in (microseconds) for
@@ -3185,7 +3247,7 @@ GstHarnessThread *
 gst_harness_stress_custom_start (GstHarness * h,
     GFunc init, GFunc callback, gpointer data, gulong sleep)
 {
-  GstHarnessCustomThread *t = g_slice_new0 (GstHarnessCustomThread);
+  GstHarnessCustomThread *t = g_new0 (GstHarnessCustomThread, 1);
   gst_harness_thread_init (&t->t,
       (GDestroyNotify) gst_harness_custom_thread_free, h, sleep);
 
@@ -3215,7 +3277,7 @@ gst_harness_stress_custom_start (GstHarness * h,
 GstHarnessThread *
 gst_harness_stress_statechange_start_full (GstHarness * h, gulong sleep)
 {
-  GstHarnessThread *t = g_slice_new0 (GstHarnessThread);
+  GstHarnessThread *t = g_new0 (GstHarnessThread, 1);
   gst_harness_thread_init (t,
       (GDestroyNotify) gst_harness_thread_free, h, sleep);
   GST_HARNESS_THREAD_START (statechange, t);
@@ -3288,7 +3350,7 @@ gst_harness_stress_push_buffer_with_cb_start_full (GstHarness * h,
     GstHarnessPrepareBufferFunc func, gpointer data, GDestroyNotify notify,
     gulong sleep)
 {
-  GstHarnessPushBufferThread *t = g_slice_new0 (GstHarnessPushBufferThread);
+  GstHarnessPushBufferThread *t = g_new0 (GstHarnessPushBufferThread, 1);
   gst_harness_thread_init (&t->t,
       (GDestroyNotify) gst_harness_push_buffer_thread_free, h, sleep);
 
@@ -3351,7 +3413,7 @@ gst_harness_stress_push_event_with_cb_start_full (GstHarness * h,
     GstHarnessPrepareEventFunc func, gpointer data, GDestroyNotify notify,
     gulong sleep)
 {
-  GstHarnessPushEventThread *t = g_slice_new0 (GstHarnessPushEventThread);
+  GstHarnessPushEventThread *t = g_new0 (GstHarnessPushEventThread, 1);
   gst_harness_thread_init (&t->t,
       (GDestroyNotify) gst_harness_push_event_thread_free, h, sleep);
 
@@ -3412,7 +3474,7 @@ gst_harness_stress_push_upstream_event_with_cb_start_full (GstHarness * h,
     GstHarnessPrepareEventFunc func, gpointer data, GDestroyNotify notify,
     gulong sleep)
 {
-  GstHarnessPushEventThread *t = g_slice_new0 (GstHarnessPushEventThread);
+  GstHarnessPushEventThread *t = g_new0 (GstHarnessPushEventThread, 1);
   gst_harness_thread_init (&t->t,
       (GDestroyNotify) gst_harness_push_event_thread_free, h, sleep);
 
@@ -3444,7 +3506,7 @@ GstHarnessThread *
 gst_harness_stress_property_start_full (GstHarness * h,
     const gchar * name, const GValue * value, gulong sleep)
 {
-  GstHarnessPropThread *t = g_slice_new0 (GstHarnessPropThread);
+  GstHarnessPropThread *t = g_new0 (GstHarnessPropThread, 1);
   gst_harness_thread_init (&t->t,
       (GDestroyNotify) gst_harness_property_thread_free, h, sleep);
 
@@ -3479,7 +3541,7 @@ gst_harness_stress_requestpad_start_full (GstHarness * h,
     GstPadTemplate * templ, const gchar * name, GstCaps * caps,
     gboolean release, gulong sleep)
 {
-  GstHarnessReqPadThread *t = g_slice_new0 (GstHarnessReqPadThread);
+  GstHarnessReqPadThread *t = g_new0 (GstHarnessReqPadThread, 1);
   gst_harness_thread_init (&t->t,
       (GDestroyNotify) gst_harness_requestpad_thread_free, h, sleep);
 
